@@ -158,6 +158,7 @@ class App:
             return parse_response(response)
 
         self.__internal__ = AppInternalMiddleware(process)  # type: ignore
+        self.__process__ = self.__internal__.app
 
         # Setup lifespan
         self.lifespan = LifespanMiddleware(
@@ -180,6 +181,8 @@ class App:
 
             self.exception_handlers[Exception] = handle_unknown_exception
 
+        self.internal_middlewares: t.List = []
+
     def __route__(self, router: Router, *prefixes: str, methods: TYPE_METHODS = None, **params):
         """Mount self as a nested application."""
         def target(request):
@@ -214,11 +217,23 @@ class App:
 
         return exc if isinstance(exc, Response) else ...
 
-    def middleware(self, md: F) -> F:
+    def middleware(self, md: F, insert_first: bool = False) -> F:
         """Register a middleware."""
         # Register as a simple middleware
         if iscoroutinefunction(md):
-            self.__internal__.bind(partial(md, self.__internal__.app))
+
+            if md not in self.internal_middlewares:
+                if insert_first:
+                    self.internal_middlewares.insert(0, md)
+                else:
+                    self.internal_middlewares.append(md)
+
+            app = self.__process__
+            for md in reversed(self.internal_middlewares):
+                app = partial(md, app)
+
+            self.__internal__.bind(app)
+
         else:
             self.lifespan.bind(md(self.lifespan.app))
 
