@@ -16,7 +16,7 @@ from yarl import URL
 from . import ASGIConnectionClosed
 from ._compat import aio_sleep, aio_spawn, aio_wait, aio_cancel, FIRST_COMPLETED
 from .response import Response, ResponseWebSocket, parse_websocket_msg
-from .utils import to_awaitable, parse_headers
+from .utils import to_awaitable, parse_headers, CIMultiDict
 from .typing import JSONType, Scope, Receive, Send, Message, ASGIApp
 
 
@@ -68,7 +68,7 @@ class TestWebSocketResponse(ResponseWebSocket):
         return self.send({'type': 'websocket.connect'})
 
     async def disconnect(self):
-        await self.send({'type': 'websocket.disconnect'})
+        await self.send({'type': 'websocket.disconnect', 'code': 1005})
         self.state = self.STATES.disconnected
 
     def send(self, msg, type='websocket.receive'):
@@ -179,8 +179,11 @@ class ASGITestClient:
         receive_from_client, send_to_app = simple_stream()
         receive_from_app, send_to_client = simple_stream()
 
+        ci_headers = CIMultiDict(headers or {})
+
         scope = self.build_scope(
-            path, headers=headers, query=query, cookies=cookies, type='websocket'
+            path, headers=ci_headers, query=query, cookies=cookies, type='websocket',
+            subprotocols=ci_headers.get('Sec-WebSocket-Protocol', '').split(','),
         )
         ws = TestWebSocketResponse(scope, receive_from_app, send_to_app)
         async with aio_spawn(self.app, scope, receive_from_client, send_to_client):
@@ -194,8 +197,8 @@ class ASGITestClient:
         return manage_lifespan(self.app, timeout=timeout)
 
     def build_scope(
-            self, path: str, headers: t.Dict = None, query: t.Union[str, t.Dict] = None,
-            cookies: t.Dict = None, **scope) -> Scope:
+            self, path: str, headers: t.Union[t.Dict, CIMultiDict] = None,
+            query: t.Union[str, t.Dict] = None, cookies: t.Dict = None, **scope) -> Scope:
         """Prepare a request scope."""
         headers = headers or {}
         headers.setdefault('Remote-Addr', '127.0.0.1')
